local AddonName, a = ...
if a.BuildFail(50000) then return end
local s = SpellFlashAddon
local x = s.UpdatedVariables
local c = BittensSpellFlashLibrary

local UnitLevel = UnitLevel
local GetPowerRegen = GetPowerRegen
local GetSpellBookItemName = GetSpellBookItemName
local GetTime = GetTime
local SPELL_POWER_BURNING_EMBERS = SPELL_POWER_BURNING_EMBERS
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS
local math = math
local select = select

c.Init(a)

------------------------------------------------------------------------ Common
local petIdentifiers = {
	[s.SpellName(c.GetID("Firebolt"), true)] = "Imp",
	[s.SpellName(c.GetID("Felbolt"), true)] = "Fel Imp",
	[s.SpellName(c.GetID("Torment"), true)] = "Voidwalker",
	[s.SpellName(c.GetID("Tongue Lash"), true)] = "Observer",
	[s.SpellName(c.GetID("Shadow Bite"), true)] = "Felhunter",
	[s.SpellName(c.GetID("Lash of Pain"), true)] = "Succubus",
}

function a.GetCurrentPet()
	for i = 1, 10000 do
		local spell = GetSpellBookItemName(i, "pet")
		if spell == nil then
			return nil
		end
		
		local pet = petIdentifiers[spell]
		if pet ~= nil then
			return pet
		end
	end
end

c.AddOptionalSpell("Dark Intent", nil, {
	Override = function()
		return c.RaidBuffNeeded(c.SPELL_POWER_BUFFS)
	end
})

c.AddOptionalSpell("Curse of the Elements", nil, {
	Debuff = c.MAGIC_VULNERABILITY_DEBUFFS
})

c.AddOptionalSpell("Soulstone", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return not s.InRaidOrParty() and c.SelfBuffNeeded("Soulstone")
	end
})

c.AddOptionalSpell("Dark Regeneration", nil, {
	CheckFirst = function()
		return s.HealthPercent("player") < 70
			or s.HealthPercent("pet") < 70
	end
})

c.AddInterrupt("Spell Lock", nil, {
	NoGCD = true,
})

c.AddInterrupt("Command Spell Lock", nil, {
	NoGCD = true,
})

c.AddInterrupt("Optical Blast", nil, {
	NoGCD = true,
})

-------------------------------------------------------------------- Affliction
local function lastTickCheck(name, earlyRefresh)
	return not c.IsCastingOrInAir(name) 
		and not c.IsQueued("Soul Swap")
		and c.GetMyDebuffDuration(name) + c.GetCastTime(name) 
			< earlyRefresh + .1
end

local function pandemicCheck(z, name, baseLength)
	if c.IsCastingOrInAir(name)
		or not s.HasSpell(c.GetID("Pandemic"))
		or c.IsQueued("Soul Swap") then
		
		return false
	end
	
	local duration = c.GetMyDebuffDuration(name)
	if duration > 0 and c.GetCooldown("Dark Soul: Misery") < duration then
		z.FlashSize = s.FlashSizePercent() / 2
	else
		z.FlashSize = nil
	end
	return duration + c.GetCastTime(name) < baseLength / 2 - 2
end

c.RegisterForFullChannels("Malefic Grasp", 4)
c.AssociateTravelTimes(.8, "Haunt")

c.AddOptionalSpell("Soulburn", "under Dark Soul: Misery", {
	CheckFirst = function()
		return c.HasBuff("Dark Soul: Misery") 
			and not a.SoulburnedDuringMisery
			and not c.HasBuff("Soulburn")
	end
})

c.AddSpell("Soulburn", "during Execute", {
	CheckFirst = function()
		if c.HasBuff("Soulburn") or c.IsCasting("Soulburn") then
			return false
		else
			return c.GetSpell("Agony"):CheckFirst()
				or c.GetSpell("Corruption"):CheckFirst()
				or c.GetSpell("Unstable Affliction"):CheckFirst()
		end
	end
})

c.AddOptionalSpell("Soul Swap", "under Soulburn", {
	CheckFirst = function()
		return c.HasBuff("Soulburn")
	end
})

c.AddSpell("Soul Swap", "during Execute", {
	CheckFirst = function()
		return c.HasBuff("Soulburn")
	end
})

c.AddOptionalSpell("Dark Soul: Misery", nil, {
	CheckFirst = function()
		return a.Shards > 0
	end
})

c.AddOptionalSpell("Grimoire: Felhunter")

c.AddSpell("Haunt", nil, {
	CheckFirst = function(z)
		local agony = c.GetMyDebuffDuration("Agony")
		if a.Shards == 0
			or not c.ShouldCastToRefresh("Haunt", "Haunt", 0, true)
			or (agony > .1 and agony - .1 < c.GetCastTime("Haunt")) then
			
			return false
		end
		
		
		if a.Shards >= s.MaxPower("player", SPELL_POWER_SOUL_SHARDS) - 1 
			or c.HasBuff("Dark Soul: Misery") then
			
			z.FlashColor = nil
			z.Continue = nil
			z.FlashSize = nil
			return true
		end
		
		local miseryCD = c.GetCooldown("Dark Soul: Misery")
		if miseryCD < 35 then
			return false
		end
		
		z.FlashColor = "yellow"
		z.Continue = true
--		local castTime = c.GetCastTime("Haunt")
--		if miseryCD < castTime then
--			z.FlashSize = nil
--		else
--			z.FlashSize = s.FlashSizePercent() / 2
--		end
		return true
	end
})

c.AddSpell("Agony", nil, {
	CheckFirst = function(z)
		return lastTickCheck("Agony", c.GetHastedTime(4))
	end
})

c.AddOptionalSpell("Agony", "with Pandemic", {
	CheckFirst = function(z)
		return pandemicCheck(z, "Agony", 24)
	end
})

c.AddSpell("Corruption", nil, {
	EarlyRefresh = 99,
	CheckFirst = function(z)
		return lastTickCheck("Corruption", z.EarlyRefresh)
	end
})

c.AddOptionalSpell("Corruption", "with Pandemic", {
	CheckFirst = function(z)
		return pandemicCheck(z, "Corruption", 18)
	end
})

c.AddSpell("Unstable Affliction", nil, {
	EarlyRefresh = 99,
	CheckFirst = function(z) 
		return lastTickCheck("Unstable Affliction", z.EarlyRefresh)
	end
})

c.AddOptionalSpell("Unstable Affliction", "with Pandemic", {
	CheckFirst = function(z) 
		return pandemicCheck(z, "Unstable Affliction", 14)
	end
})

c.AddOptionalSpell("Life Tap", "for Affliction", {
	CheckFirst = function()
		if c.HasBuff("Dark Soul: Misery") or c.HasBuff(c.BLOODLUST_BUFFS) then
			return false
		end
		
		if s.HealthPercent() <= 20 then
			return s.PowerPercent("player") < 10
		else
			return s.PowerPercent("player") < 50
		end
	end
})

-------------------------------------------------------------------- Demonology
c.AddOptionalSpell("Dark Soul: Knowledge")

c.AddOptionalSpell("Grimoire: Felguard")

c.AddOptionalSpell("Metamorphosis", nil, {
	Type = "form",
	CheckFirst = function()
		return a.Fury > 950
			or c.HasBuff("Dark Soul: Knowledge")
			or c.GetMyDebuffDuration("Corruption") < 5
	end
})

c.AddOptionalSpell("Metamorphosis", "Cancel", {
	Type = "form",
	Override = function(z)
		return a.Morphed
			and a.Fury < 650
			and not c.HasBuff("Dark Soul: Knowledge")
			and c.GetMyDebuffDuration("Corruption") > 20
	end
})

c.AddSpell("Corruption", "for Demonology", {
	FlashID = { "Corruption", "Doom" },
	CheckFirst = function()
		return not a.Morphed and not c.HasMyDebuff("Corruption")
	end
})

c.AddSpell("Doom", nil, {
	EarlyRefresh = 15,
	FlashID = { "Corruption", "Doom" },
	Override = function(z)
		return a.Morphed
			and a.Fury >= 60
			and not c.IsAuraPendingFor("Doom")
			and (c.GetMyDebuffDuration("Doom") < z.EarlyRefresh
				or (not a.DoomIsBuffed and c.HasBuff("Dark Soul: Knowledge")))
	end
})
c.ManageDotRefresh("Doom", 15)

c.AddSpell("Hand of Gul'dan", nil, {
	CheckFirst = function()
		return not a.Morphed
			and not c.HasMyDebuff("Shadowflame")
			and not c.IsAuraPendingFor("Hand of Gul'dan")
	end
})
c.RegisterAura("Hand of Gul'dan", "Shadowflame")

c.AddSpell("Touch of Chaos", "for Corruption", {
	FlashID = { "Touch of Chaos", "Shadow Bolt" },
	Override = function()
		local duration = c.GetMyDebuffDuration("Corruption")
		return a.Morphed and a.Fury >= 40 and duration > 0 and duration < 20
	end
})

c.AddSpell("Soul Fire", nil, {
	CheckFirst = function()
		local stacks = c.GetBuffStack("Molten Core")
		if c.IsCasting("Soul Fire") then
			stacks = stacks - 1
		end
		return stacks > 0
			and (not a.Morphed or a.Fury >= 80)
	end
})

c.AddSpell("Shadow Bolt", nil, {
	FlashID = { "Touch of Chaos", "Shadow Bolt" },
	CheckFirst = function()
		return not a.Morphed
	end
})

c.AddOptionalSpell("Life Tap", "under 50", {
	CheckFirst = function()
		return s.PowerPercent("player") < 50
	end
})

c.AddInterrupt("Axe Toss", nil, {
	NoGCD = true,
})

c.AddInterrupt("Super Axe Toss", nil, {
	NoGCD = true,
})

------------------------------------------------------------------- Destruction
c.AddOptionalSpell("Dark Soul: Instability")

c.AddOptionalSpell("Grimoire: Imp")

c.AddSpell("Immolate", nil, {
	FlashID = { "Immolate", "Immolate AoE" },
	CheckFirst = function()
		if c.IsCastingOrInAir("Immolate") 
			or c.IsCastingOrInAir("Immolate AoE") then
			
			return false
		end
		
		local cinCast = c.GetCastTime("Incinerate")
		local realDraft = s.Buff(c.GetID("Backdraft"), "player")
		if a.Backdraft > 0 and not realDraft then
			cinCast = cinCast / 1.3
		elseif a.Backdraft == 0 and realDraft then
			cinCast = cinCast * 1.3
		end
		return math.max(
				c.GetMyDebuffDuration("Immolate"), 
				c.GetMyDebuffDuration("Immolate AoE")) 
			< c.GetCastTime("Immolate") + cinCast + .1
	end
})

c.AddSpell("Chaos Bolt", nil, {
	CheckFirst = function()
		return a.Backdraft < 3
			and (c.GetPower(select(2, GetPowerRegen())) / s.MaxPower("player") 
					< .8
				or not s.InRaidOrParty())
			and (s.MaxPower("player", SPELL_POWER_BURNING_EMBERS) - a.Embers 
					< .5
				or c.GetBuffDuration("Dark Soul: Instability") 
					> c.GetCastTime("Chaos Bolt"))
	end
})

c.AddSpell("Conflagrate", nil, {
	CheckFirst = function()
		return a.Backdraft == 0
			and (not c.HasBuff("Backlash") 
				or c.IsCasting("Incinerate") 
				or c.IsCasting("Incinerate AoE"))
	end
})

c.AddSpell("Incinerate", nil, {
	FlashID = { "Incinerate", "Incinerate AoE" },
})

c.AddOptionalSpell("Ember Tap", nil, {
	CheckFirst = function()
		return s.HealthPercent("player") < 75
	end
})

c.AddOptionalSpell("Rain of Fire", nil, {
	MyDebuff = "Rain of Fire",
})
